@@ -1374,10 +1374,10 @@ def dump_coll_v_swap(func):
             replace_arg('rdispls', 'tmp_array + outdegree * 2 + indegree')
         else: # neighbor_alltoallw
             allocate_tmp_array("(outdegree + indegree)")
-            swap_one("indegree", "sendcounts")
-            swap_next("indegree", "outdegree", "recvcounts")
+            swap_one("outdegree", "sendcounts")
+            swap_next("outdegree", "indegree", "recvcounts")
             replace_arg('sendcounts', 'tmp_array')
-            replace_arg('recvcounts', 'tmp_array + indegree')
+            replace_arg('recvcounts', 'tmp_array + outdegree')
     # classical collectives
     elif RE.match(r'(mpi_i?reduce_scatter(_init)?\b)', func['name'], re.IGNORECASE):
         G.out.append("int n = comm_ptr->local_size;")
@@ -2249,6 +2249,14 @@ def dump_validate_op(op, dt, is_coll):
     if dt:
         G.out.append("} else {")
         G.out.append("    mpi_errno = (*MPIR_OP_HDL_TO_DTYPE_FN(%s)) (%s);" % (op, dt))
+        # check predefined datatype and replace with basic_type if necessary
+        G.out.append("    if (mpi_errno != MPI_SUCCESS) {")
+        G.out.append("        MPI_Datatype alt_dt = MPIR_Op_get_alt_datatype(%s, %s);" % (op, dt))
+        G.out.append("        if (alt_dt != MPI_DATATYPE_NULL) {")
+        G.out.append("            %s = alt_dt;" % dt)
+        G.out.append("            mpi_errno = MPI_SUCCESS;")
+        G.out.append("        }")
+        G.out.append("    }")
     G.out.append("}")
     dump_error_check("")
 
