@@ -13,7 +13,7 @@
 
 categories :
     - name : REQUEST
-      description : A category for requests mangement variables
+      description : A category for requests management variables
 
 cvars:
     - name        : MPIR_CVAR_REQUEST_POLL_FREQ
@@ -61,6 +61,10 @@ typedef enum MPIR_Request_kind_t {
     MPIR_REQUEST_KIND__RECV,
     MPIR_REQUEST_KIND__PREQUEST_SEND,
     MPIR_REQUEST_KIND__PREQUEST_RECV,
+    MPIR_REQUEST_KIND__PREQUEST_COLL,
+    MPIR_REQUEST_KIND__PART_SEND,       /* Partitioned send req returned to user */
+    MPIR_REQUEST_KIND__PART_RECV,       /* Partitioned recv req returned to user */
+    MPIR_REQUEST_KIND__PART,    /* Partitioned pt2pt internal reqs */
     MPIR_REQUEST_KIND__GREQUEST,
     MPIR_REQUEST_KIND__COLL,
     MPIR_REQUEST_KIND__MPROBE,  /* see NOTE-R1 */
@@ -124,6 +128,9 @@ typedef struct MPIR_Grequest_class {
     struct MPIR_Grequest_class *next;
 } MPIR_Grequest_class;
 
+extern MPIR_Grequest_class MPIR_Grequest_class_direct[];
+extern MPIR_Object_alloc_t MPIR_Grequest_class_mem;
+
 #define MPIR_Request_extract_status(request_ptr_, status_)              \
     {                                                                   \
         if ((status_) != MPI_STATUS_IGNORE)                             \
@@ -143,6 +150,13 @@ typedef struct MPIR_Grequest_class {
 
 #define MPIR_Request_is_complete(req_) (MPIR_cc_is_complete((req_)->cc_ptr))
 
+/* types of sched structure used in persistent collective */
+enum MPIR_sched_type {
+    MPIR_SCHED_INVALID,
+    MPIR_SCHED_NORMAL,
+    MPIR_SCHED_GENTRAN
+};
+
 /*S
   MPIR_Request - Description of the Request data structure
 
@@ -190,25 +204,42 @@ struct MPIR_Request {
             MPIR_Errflag_t errflag;
             MPII_Coll_req_t coll;
         } nbc;                  /* kind : MPIR_REQUEST_KIND__COLL */
-#if defined HAVE_DEBUGGER_SUPPORT
         struct {
-            struct MPIR_Sendq *dbg_next;
-        } send;                 /* kind : MPID_REQUEST_SEND */
-#endif                          /* HAVE_DEBUGGER_SUPPORT */
-        struct {
-#if defined HAVE_DEBUGGER_SUPPORT
-            struct MPIR_Sendq *dbg_next;
-#endif                          /* HAVE_DEBUGGER_SUPPORT */
             /* Persistent requests have their own "real" requests */
             struct MPIR_Request *real_request;
-        } persist;              /* kind : MPID_PREQUEST_SEND or MPID_PREQUEST_RECV */
+            MPIR_TSP_sched_t sched;
+        } persist;              /* kind : MPIR_REQUEST_KIND__PREQUEST_SEND or MPIR_REQUEST_KIND__PREQUEST_RECV */
+        struct {
+            struct MPIR_Request *real_request;
+            enum MPIR_sched_type sched_type;
+            void *sched;
+            MPII_Coll_req_t coll;
+        } persist_coll;         /* kind : MPIR_REQUEST_KIND__PREQUEST_COLL */
+        struct {
+            int partitions;     /* Needed for parameter error check */
+            MPL_atomic_int_t active_flag;       /* flag indicating whether in a start-complete active period.
+                                                 * Value is 0 or 1. */
+        } part;                 /* kind : MPIR_REQUEST_KIND__PART_SEND or MPIR_REQUEST_KIND__PART_RECV */
+        struct {
+            MPIR_Win *win;
+        } rma;                  /* kind : MPIR_REQUEST_KIND__RMA */
     } u;
 
+#if defined HAVE_DEBUGGER_SUPPORT
+    struct MPIR_Debugq *send;
+    struct MPIR_Debugq *recv;
+    struct MPIR_Debugq *unexp;
+#endif                          /* HAVE_DEBUGGER_SUPPORT */
+
+    struct MPIR_Request *next, *prev;
+
     /* Other, device-specific information */
 #ifdef MPID_DEV_REQUEST_DECL
      MPID_DEV_REQUEST_DECL
 #endif
 };
+int MPIR_Persist_coll_start(MPIR_Request * request);
+void MPIR_Persist_coll_free_cb(MPIR_Request * request);
 
 /* Multiple Request Pools
  * Request objects creation and freeing is in a hot path. Multiple pools allow different
@@ -237,7 +268,7 @@ struct MPIR_Request {
 #define MPIR_REQUEST_NUM_POOLS REQUEST_POOL_MAX
 #define MPIR_REQUEST_PREALLOC 8
 
-#define MPIR_REQUEST_POOL(req_) (((req_)->handle & REQUEST_POOL_MASK) >> REQUEST_POOL_SHIFT);
+#define MPIR_REQUEST_POOL(req_) (((req_)->handle & REQUEST_POOL_MASK) >> REQUEST_POOL_SHIFT)
 
 extern MPIR_Request MPIR_Request_builtins[MPIR_REQUEST_BUILTIN_COUNT];
 extern MPIR_Object_alloc_t MPIR_Request_mem[MPIR_REQUEST_NUM_POOLS];
@@ -285,7 +316,29 @@ static inline void MPIR_Request_register_pool_lock(int pool, MPID_Thread_mutex_t
 static inline int MPIR_Request_is_persistent(MPIR_Request * req_ptr)
 {
     return (req_ptr->kind == MPIR_REQUEST_KIND__PREQUEST_SEND ||
-            req_ptr->kind == MPIR_REQUEST_KIND__PREQUEST_RECV);
+            req_ptr->kind == MPIR_REQUEST_KIND__PREQUEST_RECV ||
+            req_ptr->kind == MPIR_REQUEST_KIND__PREQUEST_COLL);
+}
+
+static inline int MPIR_Request_is_partitioned(MPIR_Request * req_ptr)
+{
+    return (req_ptr->kind == MPIR_REQUEST_KIND__PART_SEND ||
+            req_ptr->kind == MPIR_REQUEST_KIND__PART_RECV);
+}
+
+static inline int MPIR_Part_request_is_active(MPIR_Request * req_ptr)
+{
+    return MPL_atomic_load_int(&req_ptr->u.part.active_flag);
+}
+
+static inline void MPIR_Part_request_inactivate(MPIR_Request * req_ptr)
+{
+    MPL_atomic_store_int(&req_ptr->u.part.active_flag, 0);
+}
+
+static inline void MPIR_Part_request_activate(MPIR_Request * req_ptr)
+{
+    MPL_atomic_store_int(&req_ptr->u.part.active_flag, 1);
 }
 
 /* Return whether a request is active.
@@ -296,8 +349,18 @@ static inline int MPIR_Request_is_active(MPIR_Request * req_ptr)
 {
     if (req_ptr == NULL)
         return 0;
-    else
-        return (!MPIR_Request_is_persistent(req_ptr) || (req_ptr)->u.persist.real_request != NULL);
+    else {
+        switch (req_ptr->kind) {
+            case MPIR_REQUEST_KIND__PREQUEST_SEND:
+            case MPIR_REQUEST_KIND__PREQUEST_RECV:
+                return (req_ptr)->u.persist.real_request != NULL;
+            case MPIR_REQUEST_KIND__PART_SEND:
+            case MPIR_REQUEST_KIND__PART_RECV:
+                return MPIR_Part_request_is_active(req_ptr);
+            default:
+                return 1;       /* regular request is always active */
+        }
+    }
 }
 
 #define MPIR_REQUESTS_PROPERTY__NO_NULL        (1 << 1)
@@ -309,8 +372,10 @@ static inline int MPIR_Request_is_active(MPIR_Request * req_ptr)
 
 /* NOTE: Pool-specific request creation is unsafe unless under global thread granularity.
  */
-static inline MPIR_Request *MPIR_Request_create_from_pool(MPIR_Request_kind_t kind, int pool)
+static inline MPIR_Request *MPIR_Request_create_from_pool(MPIR_Request_kind_t kind, int pool,
+                                                          int ref_count)
 {
+    MPIR_Assert(ref_count >= 1);
     MPIR_Request *req;
 
 #ifdef MPICH_DEBUG_MUTEX
@@ -318,59 +383,77 @@ static inline MPIR_Request *MPIR_Request_create_from_pool(MPIR_Request_kind_t ki
 #endif
     req = MPIR_Handle_obj_alloc_unsafe(&MPIR_Request_mem[pool],
                                        REQUEST_NUM_BLOCKS, REQUEST_NUM_INDICES);
-    if (req != NULL) {
-        /* Patch the handle for pool index. */
-        req->handle |= (pool << REQUEST_POOL_SHIFT);
-    }
+    if (req == NULL)
+        goto fn_fail;
 
-    if (req != NULL) {
-        MPL_DBG_MSG_P(MPIR_DBG_REQUEST, VERBOSE, "allocated request, handle=0x%08x", req->handle);
+    /* Patch the handle for pool index. */
+    req->handle |= (pool << REQUEST_POOL_SHIFT);
+
+    MPL_DBG_MSG_P(MPIR_DBG_REQUEST, VERBOSE, "allocated request, handle=0x%08x", req->handle);
 #ifdef MPICH_DBG_OUTPUT
-        /*MPIR_Assert(HANDLE_GET_MPI_KIND(req->handle) == MPIR_REQUEST); */
-        if (HANDLE_GET_MPI_KIND(req->handle) != MPIR_REQUEST) {
-            int mpi_errno;
-            mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL,
-                                             __func__, __LINE__, MPI_ERR_OTHER,
-                                             "**invalid_handle", "**invalid_handle %d",
-                                             req->handle);
-            MPID_Abort(MPIR_Process.comm_world, mpi_errno, -1, NULL);
-        }
-#endif
-        /* FIXME: This makes request creation expensive.  We need to
-         * trim this to the basics, with additional setup for
-         * special-purpose requests (think base class and
-         * inheritance).  For example, do we really* want to set the
-         * kind to UNDEFINED? And should the RMA values be set only
-         * for RMA requests? */
-        MPIR_Object_set_ref(req, 1);
-        req->kind = kind;
-        MPIR_cc_set(&req->cc, 1);
-        req->cc_ptr = &req->cc;
-
-        req->completion_notification = NULL;
-
-        req->status.MPI_ERROR = MPI_SUCCESS;
-        MPIR_STATUS_SET_CANCEL_BIT(req->status, FALSE);
-
-        req->comm = NULL;
-
-        switch (kind) {
-            case MPIR_REQUEST_KIND__SEND:
-                MPII_REQUEST_CLEAR_DBG(req);
-                break;
-            case MPIR_REQUEST_KIND__COLL:
-                req->u.nbc.errflag = MPIR_ERR_NONE;
-                break;
-            default:
-                break;
-        }
-
-        MPID_Request_create_hook(req);
-    } else {
-        /* FIXME: This fails to fail if debugging is turned off */
-        MPL_DBG_MSG(MPIR_DBG_REQUEST, TYPICAL, "unable to allocate a request");
+    /*MPIR_Assert(HANDLE_GET_MPI_KIND(req->handle) == MPIR_REQUEST); */
+    if (HANDLE_GET_MPI_KIND(req->handle) != MPIR_REQUEST) {
+        int mpi_errno;
+        mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL,
+                                         __func__, __LINE__, MPI_ERR_OTHER,
+                                         "**invalid_handle", "**invalid_handle %d", req->handle);
+        MPID_Abort(MPIR_Process.comm_world, mpi_errno, -1, NULL);
     }
+#endif
+    /* FIXME: This makes request creation expensive.  We need to
+     * trim this to the basics, with additional setup for
+     * special-purpose requests (think base class and
+     * inheritance).  For example, do we really* want to set the
+     * kind to UNDEFINED? And should the RMA values be set only
+     * for RMA requests? */
+    MPIR_Object_set_ref(req, ref_count);
+    req->kind = kind;
+    MPIR_cc_set(&req->cc, 1);
+    req->cc_ptr = &req->cc;
 
+    req->completion_notification = NULL;
+
+    req->status.MPI_ERROR = MPI_SUCCESS;
+    MPIR_STATUS_SET_CANCEL_BIT(req->status, FALSE);
+
+    req->comm = NULL;
+
+    switch (kind) {
+        case MPIR_REQUEST_KIND__COLL:
+            req->u.nbc.errflag = MPIR_ERR_NONE;
+            req->u.nbc.coll.host_sendbuf = NULL;
+            req->u.nbc.coll.host_recvbuf = NULL;
+            req->u.nbc.coll.datatype = MPI_DATATYPE_NULL;
+            break;
+        case MPIR_REQUEST_KIND__PREQUEST_COLL:
+            req->u.persist_coll.coll.host_sendbuf = NULL;
+            req->u.persist_coll.coll.host_recvbuf = NULL;
+            req->u.persist_coll.coll.datatype = MPI_DATATYPE_NULL;
+        default:
+            break;
+    }
+    MPII_REQUEST_CLEAR_DBG(req);
+
+    MPID_Request_create_hook(req);
+
+  fn_exit:
+    return req;
+  fn_fail:
+    MPIR_Assert(req != NULL);
+    /* TODO - Obviously this is a bad solution, but no one had the stomach to make the larger change
+     * in the entire codebase to do a better job. */
+    goto fn_exit;
+}
+
+/* Useful for lockless MT model */
+static inline MPIR_Request *MPIR_Request_create_from_pool_safe(MPIR_Request_kind_t kind, int pool,
+                                                               int ref_count)
+{
+    MPIR_Request *req;
+
+    MPID_THREAD_CS_ENTER(VCI, (*(MPID_Thread_mutex_t *) MPIR_Request_mem[pool].lock));
+    req = MPIR_Request_create_from_pool(kind, pool, ref_count);
+    MPID_THREAD_CS_EXIT(VCI, (*(MPID_Thread_mutex_t *) MPIR_Request_mem[pool].lock));
     return req;
 }
 
@@ -380,7 +463,7 @@ static inline MPIR_Request *MPIR_Request_create(MPIR_Request_kind_t kind)
     MPIR_Request *req;
     MPID_THREAD_CS_ENTER(POBJ, MPIR_THREAD_POBJ_HANDLE_MUTEX);
     MPID_THREAD_CS_ENTER(VCI, (*(MPID_Thread_mutex_t *) MPIR_Request_mem[0].lock));
-    req = MPIR_Request_create_from_pool(kind, 0);
+    req = MPIR_Request_create_from_pool(kind, 0, 1);
     MPID_THREAD_CS_EXIT(POBJ, MPIR_THREAD_POBJ_HANDLE_MUTEX);
     MPID_THREAD_CS_EXIT(VCI, (*(MPID_Thread_mutex_t *) MPIR_Request_mem[0].lock));
     return req;
@@ -464,14 +547,20 @@ static inline void MPIR_Request_free_with_safety(MPIR_Request * req, int need_sa
             MPL_free(req->u.ureq.greq_fns);
         }
 
+        if (req->kind == MPIR_REQUEST_KIND__SEND) {
+            MPII_SENDQ_FORGET(req);
+        } else if (req->kind == MPIR_REQUEST_KIND__RECV) {
+            MPII_RECVQ_FORGET(req);
+        }
+
         MPID_Request_destroy_hook(req);
 
         if (need_safety) {
             MPID_THREAD_CS_ENTER(POBJ, MPIR_THREAD_POBJ_HANDLE_MUTEX);
-            MPIR_Handle_obj_free_unsafe(&MPIR_Request_mem[pool], req);
+            MPIR_Handle_obj_free_unsafe(&MPIR_Request_mem[pool], req, /* not info */ FALSE);
             MPID_THREAD_CS_EXIT(POBJ, MPIR_THREAD_POBJ_HANDLE_MUTEX);
         } else {
-            MPIR_Handle_obj_free_unsafe(&MPIR_Request_mem[pool], req);
+            MPIR_Handle_obj_free_unsafe(&MPIR_Request_mem[pool], req, /* not info */ FALSE);
         }
     }
     if (need_safety) {
@@ -496,7 +585,7 @@ MPL_STATIC_INLINE_PREFIX void MPIR_Request_free(MPIR_Request * req)
 }
 
 /* Requests that are not created inside device (general requests, nonblocking collective
- * requests such as sched, gentran, hcoll) should call MPIR_Request_complete.
+ * requests such as sched, tsp, hcoll) should call MPIR_Request_complete.
  * MPID_Request_complete are called inside device critical section, therefore, potentially
  * are unsafe to call outside the device. (NOTE: this will come into effect with ch4 multi-vci.)
  */
@@ -521,11 +610,6 @@ MPL_STATIC_INLINE_PREFIX int MPIR_Request_completion_processing_fastpath(MPI_Req
     MPIR_Assert(request_ptr->kind == MPIR_REQUEST_KIND__SEND ||
                 request_ptr->kind == MPIR_REQUEST_KIND__RECV);
 
-    if (request_ptr->kind == MPIR_REQUEST_KIND__SEND) {
-        /* FIXME: are Ibsend requests added to the send queue? */
-        MPII_SENDQ_FORGET(request_ptr);
-    }
-
     /* the completion path for SEND and RECV is the same at this time, modulo
      * the SENDQ hook above */
     mpi_errno = request_ptr->status.MPI_ERROR;
@@ -564,17 +648,6 @@ int MPIR_Grequest_cancel(MPIR_Request * request_ptr, int complete);
 int MPIR_Grequest_query(MPIR_Request * request_ptr);
 int MPIR_Grequest_free(MPIR_Request * request_ptr);
 
-void MPIR_Grequest_complete(MPIR_Request * request_ptr);
-int MPIR_Grequest_start(MPI_Grequest_query_function * query_fn,
-                        MPI_Grequest_free_function * free_fn,
-                        MPI_Grequest_cancel_function * cancel_fn,
-                        void *extra_state, MPIR_Request ** request_ptr);
-int MPIX_Grequest_start_impl(MPI_Grequest_query_function *,
-                             MPI_Grequest_free_function *,
-                             MPI_Grequest_cancel_function *,
-                             MPIX_Grequest_poll_function *,
-                             MPIX_Grequest_wait_function *, void *, MPIR_Request **);
-
 /* These routines below are helpers for the Extended generalized requests. */
 
 MPL_STATIC_INLINE_PREFIX int MPIR_Request_has_poll_fn(MPIR_Request * request_ptr)
@@ -611,6 +684,9 @@ MPL_STATIC_INLINE_PREFIX int MPIR_Grequest_poll(MPIR_Request * request_ptr, MPI_
     return mpi_errno;
 }
 
+/* local request array size in MPI_Start_all and MPI_{Test,Wait}{all,any,some} */
+#define MPIR_REQUEST_PTR_ARRAY_SIZE 64
+
 int MPIR_Test_state(MPIR_Request * request, int *flag, MPI_Status * status,
                     MPID_Progress_state * state);
 int MPIR_Testall_state(int count, MPIR_Request * request_ptrs[], int *flag,
@@ -647,7 +723,16 @@ int MPIR_Waitsome_impl(int incount, MPIR_Request * request_ptrs[],
 int MPIR_Test(MPI_Request * request, int *flag, MPI_Status * status);
 int MPIR_Testall(int count, MPI_Request array_of_requests[], int *flag,
                  MPI_Status array_of_statuses[]);
+int MPIR_Testany(int count, MPI_Request array_of_requests[], MPIR_Request * request_ptrs[],
+                 int *indx, int *flag, MPI_Status * status);
+int MPIR_Testsome(int incount, MPI_Request array_of_requests[], MPIR_Request * request_ptrs[],
+                  int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);
 int MPIR_Wait(MPI_Request * request, MPI_Status * status);
 int MPIR_Waitall(int count, MPI_Request array_of_requests[], MPI_Status array_of_statuses[]);
+int MPIR_Waitany(int count, MPI_Request array_of_requests[], MPIR_Request * request_ptrs[],
+                 int *indx, MPI_Status * status);
+int MPIR_Waitsome(int incount, MPI_Request array_of_requests[], MPIR_Request * request_ptrs[],
+                  int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);
+int MPIR_Parrived(MPIR_Request * request_ptr, int partition, int *flag);
 
 #endif /* MPIR_REQUEST_H_INCLUDED */
